
from ovirtsdk.api import API
from ovirtsdk.xml import params

import scaleconfig
import logging

from time import gmtime, strftime, sleep
from datetime import datetime

logging.basicConfig(filename='messages_scale.log',level=logging.DEBUG)
LOGGER = logging.getLogger(__name__)

VERSION_DC = params.Version(major=scaleconfig.DEFAULT_MAJOR_DC_VERSION, minor=scaleconfig.DEFAULT_MINOR_DC_VERSION)
VERSION_CLUSTER = params.Version(major=scaleconfig.DEFAULT_MAJOR_CLUSTER_VERSION, minor=scaleconfig.DEFAULT_MINOR_CLUSTER_VERSION)

#API = API(url=scaleconfig.URL, username=scaleconfig.USER_NAME, password=scaleconfig.USER_PASSWORD)
API = API(url=scaleconfig.URL, username=scaleconfig.USER_NAME, password=scaleconfig.USER_PASSWORD, debug=True) #### In DEBUG mode

MB = 1024*1024
GB = 1024*MB

def timeIt(func):
    '''
    decorator for logging results in a clean manner and timing function runs
    '''
    def inner(*args,**kwargs):
        LOGGER.info("=====================================================================================================================")
        LOGGER.info(timestampScale() +  "Entering function: %s, with params: args = %s, kwargs = %s" % (func.__name__, args, kwargs))
        LOGGER.info("--------------------------------------------------------------------------------------------------------------------")
        start_time = datetime.now()
        result = func(*args, **kwargs)
        end_time = datetime.now()
        total_time = end_time - start_time

        LOGGER.info(timestampScale() +  "Exiting function: %s" % func.__name__)
        LOGGER.info(timestampScale() +  "Result was: %s" % result)
        LOGGER.info("--------------------------------------------------------------------------------------------------------------------")
        LOGGER.info(timestampScale() +  "Total time was: %s" % total_time)
        LOGGER.info("=====================================================================================================================")
    return inner

# ---------------------------------------------------------------------------------------

def timestampScale():
#    print 'TimeStamp ====> ' + strftime("%Y-%m-%d %H:%M:%S", gmtime())
#    return str(" <== TimeStamp ==> " +  ("%Y-%m-%d %H:%M:%S.%f", gmtime()) + ":")
    now = datetime.now()
    return str(" <== TimeStamp ==> " +  now.strftime("%H:%M:%S.%f") + ":")


def createDataCenter(name, storageType=scaleconfig.MAIN_STORAGE_TYPE, version=VERSION_DC):
    LOGGER.info(timestampScale() + "Creating data center '%s'" % name)
    API.datacenters.add(params.DataCenter(                                          
                name=name,
                storage_type=storageType,
                version=version))
    dc = API.datacenters.get(name)
    return dc is not None
    

def removeDataCenter(name):
    """ Removes datacenter """
    dc = API.datacenters.get(name)
    if dc is not None:
        LOGGER.info(timestampScale() + ":Removing datacenter '%s'" % name)
        dc.delete()
        return dc is not None, "Can't remove datacenter"
#        assert updateObject(dc) is None, "Can't remove datacenter"
#        if updateObject(dc) is not None: #, "Can't remove datacenter"
#            raise Execption("unable to remove dataCenter %s" % name)

def createCluster(name, datacenterName,
                    cpu_type=scaleconfig.HOST_CPU_TYPE_AMD, version=VERSION_CLUSTER):
    """ Creates cluster """
    LOGGER.info(timestampScale() + "create_cluster")
    dc = API.datacenters.get(datacenterName)
    API.clusters.add(params.Cluster(
                name=name,
                cpu=params.CPU(id=cpu_type),
                data_center=dc,
                version=version))
    cluster = API.clusters.get(name)
    LOGGER.info(timestampScale() + "Creating cluster '%s'" % name)
    return cluster is not None

def removeCluster(name):
    """ Removes cluster """
    cluster = API.clusters.get(name)
    if cluster is not None:
        cluster.delete()
        LOGGER.info(timestampScale() + "Removing cluster '%s'" % name)
#        assert updateObject(cluster) is None, "Can't remove cluster"
        assert cluster is not None, "Can't remove cluster"

def createHost(clusterName, hostName, hostAddress, hostPassword):
    """ create host """
    msg = "Installing host '%s' on '%s'"
    LOGGER.info(timestampScale() + msg % (hostAddress, clusterName))
    LOGGER.info(timestampScale() + "ClusterName=%s, hostName=%s, hostAdress=%s, hostPassword=%s" % (clusterName, hostName, hostAddress, hostPassword))

    cluster = API.clusters.get(clusterName)
    API.hosts.add(params.Host(
            name=hostName,
            address=hostAddress,
            cluster=cluster,
            root_password=hostPassword))
    host = API.hosts.get(hostName)
    return host is not None

#    waitForState(host, states.host.up,
#            failStates = states.host.install_failed,
#            timeout = config.HOST_INSTALL_TIMEOUT,
#            restoreState=states.host.non_operational)

#################### STORAGES #################################################
#def createNfsStorage(storageName, storageType='data',
#                    address=config.NFS_STORAGE_ADDRESS,
#                    path=config.NFS_STORAGE_PATH,
#                    datacenter=config.MAIN_DC_NAME,
#                    host=config.MAIN_HOST_NAME):
def createNfsStorage(storageName, 
                     address, 
                     path, 
                     datacenter, 
                     host, storageType='data'):
    """ Creates NFS storage, but does not attach it.

    Does not test if the storage already exists, so that it can be used in
    negative tests (the error will be produced by the sdk, not by this
    function.

    :param storageName:     (string) name of the storage domain to create
    :param storageType:     (string) either 'data', 'export' or 'iso'
    :param address:         (string) IP address of the storage
    :param path:            (string) path to the storage on the server given by
                            `address`, for example `'/mnt/nfs/mystorage'`
    :param datacenter:      (string) name of the datacenter where it will
                            be created
    :param host:            (string) name of the host to use
    """

    dc = API.datacenters.get(datacenter)
#    sd = API.storagedomains.get(storageName)

    storageParams = params.Storage(type_='nfs',
            address = address, path = path)
    sdParams = params.StorageDomain(name=storageName,
                data_center=dc, storage_format='v3', type_=storageType,
                host=API.hosts.get(host), storage = storageParams)

    LOGGER.info(timestampScale() + "Creating NFS storage with name '%s' at host '%s'" %
            (storageName, host))
    LOGGER.info(timestampScale() + "IP/Path of NFS: %s:%s" %(address, path))
    storage = API.storagedomains.add(sdParams)
    storage = API.storagedomains.get(storage.get_name())
    return storage is not None, "Failed to create storage"
#    return storage.get_name()
'''
def removeAllFromSD(sdName):
    """ Removes vms and temapltes from SD """
    sd = API.storagedomains.get(sdName)
    for vm in sd.vms.list():
        removeVmObject(vm)
    for tmp in sd.templates.list():
        removeTemplate(tmp.get_name())
'''

#def createIscsiStorage(storageName, storageType='data',
#                    address=config.LUN_ADDRESS,
#                    target=config.LUN_TARGET,
#                    guid=config.LUN_GUID,
#                    datacenter=config.MAIN_DC_NAME,
#                    host=config.MAIN_HOST_NAME):
def createIscsiStorage(storageName,
                    address,
                    target,
                    guid,
                    datacenter,
                    host, storageType='data'):
    
    """ Creates iSCSI storage, but does not attach it.

    Does not test if the storage already exists, so that it can be used in
    negative tests (the error will be produced by the sdk, not by this
    function.

    .. seealso:: :func:`createNfsStorage`
    """
    dc = API.datacenters.get(datacenter)

    logicalUnit = params.LogicalUnit(id=guid,
            address=address, port=3260, target=target)
    storageParams = params.Storage(type_='iscsi',
            volume_group=params.VolumeGroup(logical_unit=[logicalUnit]))

    sdParams = params.StorageDomain(name=storageName,
                data_center=dc, storage_format='v1', type_=storageType,
                host=API.hosts.get(host), storage = storageParams)

    LOGGER.info(timestampScale() + "Creating iSCSI storage with name '%s'" % (storageName))
    API.storagedomains.add(sdParams)
    storage = API.storagedomains.get(storageName)
    return storage is not None, "Failed to create storage"

####################################################################################################################

#### Attach Storage Domain ###
#try:
#    if api.datacenters.get(name=Data_Center__name).storagedomains.add(api.storagedomains.get(name=Storage_DOMAIN__name)):
#        print 'FC Storage Domain was attached successfully'
#except Exception as e:
#    print 'Failed to attach FC Storage Domain:\n%s' % str(e)


####################################################################################################################

def createFcpStorage(storageName,
                    lunid,
                    datacenter,
                    host, storageType='data'):
    
    """ Creates FCP storage, but does not attach it.

    Does not test if the storage already exists, so that it can be used in
    negative tests (the error will be produced by the sdk, not by this
    function.

    .. seealso:: :func:`createNfsStorage`
    """
    
    dc = API.datacenters.get(datacenter)

    logicalUnit = params.LogicalUnit(id=lunid)
    storageParams = params.Storage(type_='fcp',
            volume_group=params.VolumeGroup(logical_unit=[logicalUnit]))

    sdParams = params.StorageDomain(name=storageName,
                data_center=dc, storage_format='v3', type_=storageType,
                host=API.hosts.get(host), storage = storageParams)

    LOGGER.info(timestampScale() + "Creating FCP storage with name '%s'" % (storageName))
    API.storagedomains.add(sdParams)
    storage = API.storagedomains.get(storageName)
    return storage is not None, "Failed to create storage"
    
'''
def deactivateActivateByStateObject(storage, storageInDc, state, jmp,
        datacenter=config.MAIN_DC_NAME):
    if  jmp or (storage.status is None and storageInDc is not None and \
        storageInDc.status is not None and \
        storageInDc.status.state != state):
            storageInDc.deactivate()
            waitForState(storageInDc, states.storage.maintenance, timeout=10*60)
            storageInDc.activate()
            waitForState(storageInDc, states.storage.active, timeout=10*60)

def deactivateActivate(storageName, datacenter=config.MAIN_DC_NAME):
    deactivateActivateByState(storageName=storageName, state=states.storage.active, jmp=False,
            datacenter=datacenter)

def deactivateActivateByState(storageName, state, jmp, datacenter=config.MAIN_DC_NAME):
    dc = API.datacenters.get(datacenter)
    storage = API.storagedomains.get(storageName)

    storageInDc = dc.storagedomains.get(storageName)

    if  jmp or (storage.status is None and storageInDc is not None and \
        storageInDc.status is not None and \
        storageInDc.status.state != state):
            storageInDc.deactivate()
            waitForState(storageInDc, states.storage.maintenance, timeout=10*60)
            storageInDc.activate()
            waitForState(storageInDc, states.storage.active, timeout=10*60)
'''

#def attachActivateStorage(storageName, isMaster=False,
#                            datacenter=config.MAIN_DC_NAME):
def attachActivateStorage(storageName, 
                          datacenter, isMaster=False):
    """ Attach and activate a storage domain.

    Will not check if the storage is already active/attached, so it can be
    used in negative tests (the error will be from the sdk, not from here)
    """

    LOGGER.info(timestampScale() + "Attaching storage '%s' to data center" % (storageName))
    dc = API.datacenters.get(datacenter)
    storage = API.storagedomains.get(storageName)
    if dc.storagedomains.get(storageName) is not None:
        return

    dc.storagedomains.add(storage)
    storage = dc.storagedomains.get(storageName)
    return storage is not None, "Failed to attach storage to datacenter"

    # the main storage gets activated on it's own, no need to call activate()
    if not isMaster:
        storage.activate()
#    waitForState(storage, states.storage.active, timeout=10*60)

'''
def removeNonMasterStorage(storageName,
                            datacenter=config.MAIN_DC_NAME,
                            host=config.MAIN_HOST_NAME,
                            destroy=False):
    """ Deactivate, detach and remove a non-master storage domain.  """
    dc = API.datacenters.get(name=datacenter)
    storage = API.storagedomains.get(storageName)
    if storage is None:
        LOGGER.warning("SD '%s' not exists." % storageName)
        return

    doFormat = False if storage.get_type() == 'iso' else True

    if isStorageAttached(storageName):
        s = dc.storagedomains.get(storageName)
        if s.status.state != states.storage.inactive and \
            s.status.state != states.storage.maintenance:
                LOGGER.info("Deactivating storage")
                s.deactivate()
                waitForState(s, [states.storage.inactive, states.storage.maintenance])
        if not destroy:
            LOGGER.info("Detaching storage from data center")
            s.delete()
            s = API.storagedomains.get(storageName)
            waitForState(s, states.storage.unattached)

    LOGGER.info("Deleting storage '%s'" % (storageName))
    param = params.StorageDomain(name=storageName,
            host=params.Host(name=host), format=doFormat,
            destroy=destroy)
    storage.delete(param)
    storage = API.storagedomains.get(storageName)
    assert storage is None, "Failed to remove SD '%s'" % (storageName)

def deactivateMasterStorage(storageName=config.MAIN_STORAGE_NAME,
                        datacenter=config.MAIN_DC_NAME,
                        host=config.MAIN_HOST_NAME):
    """ Deactivates the master storage domain and its datacenter.

    Fails if the storage doesn't exist (and therefore will not remove the
    datacenter).
    """
    dc = API.datacenters.get(name=datacenter)
    assert dc is not None
    sd = dc.storagedomains

    storage = sd.get(storageName)
    assert storage is not None

    if storage.status.state == states.storage.inactive:
        LOGGER.warning("Master storage in maintenance before removal")
    else:
        LOGGER.info("Deactivating storage")
        storage.deactivate()
        waitForState(storage, states.storage.maintenance)

def removeMasterStorage(storageName=config.MAIN_STORAGE_NAME,
                        datacenter=config.MAIN_DC_NAME,
                        host=config.MAIN_HOST_NAME):
    """ Deactivates and removes the master storage domain and its datacenter.

    Fails if the storage doesn't exist (and therefore will not remove the
    datacenter).
    """
    dc = API.datacenters.get(name=datacenter)
    assert dc is not None
    sd = dc.storagedomains

    storage = sd.get(storageName)
    assert storage is not None

    LOGGER.info("Storage state: %s" %storage.status.state)
    try:
        waitForState(storage, states.storage.active)
    except:
        LOGGER.warning("Storage state: %s" %storage.status.state)
        try:
            storage.activate()
            waitForState(storage, states.storage.active)
        except:
            LOGGER.warning("Storage was not activated.")
    if storage.status.state == states.storage.inactive:
        LOGGER.warning("Master storage in maintenance before removal")
    else:
        LOGGER.info("Deactivating storage")
        storage.deactivate()
        waitForState(storage, states.storage.maintenance)

    LOGGER.info("Deleting data center")
    dc.delete()
    assert updateObject(dc) is None, "Failed to remove DC"

    storage = API.storagedomains.get(storageName)
    assert storage.status.state == states.storage.unattached, \
        "Storage '%s' was not detached from DC" % storageName
    LOGGER.info("Deleting main storage")
    storage.delete(params.StorageDomain(name=storageName,
        host=params.Host(name=host), format=True))
    assert API.storagedomains.get(storageName) is None, \
        "Cannot remove master storage"


def isStorageAttached(storageName, datacenter=config.MAIN_DC_NAME):
    """ Return True if storage domain is attached to a datacenter.

    :return:    (bool) True if it is attached (even if it is locked or
                unreachable), False if it doesn't exist or if it is unattached.
    """
    dc = API.datacenters.get(name=datacenter)
    storage = API.storagedomains.get(storageName)
    if storage is None: # storage doesn't even exist
        return False

    storageInDc = dc.storagedomains.get(storageName)
    if  storage.status is None and storageInDc is not None and \
        storageInDc.status is not None:
            return True
    else:
            return False

def detachAttachSD(storageName=config.MAIN_STORAGE_NAME,
                   datacenter=config.MAIN_DC_NAME):
    """ Detach and attach SD from DC """
    if isStorageAttached(storageName):
        LOGGER.info("Deactivating/activating '%s'" % storageName)
        deactivateMasterStorage(storageName=storageName,
                datacenter=datacenter,
                host=config.MAIN_HOST_NAME)
        attachActivateStorage(storageName)
    else:
        LOGGER.info("Activation/deactivating '%s'" % storageName)
        attachActivateStorage(storageName)
        deactivateMasterStorage(storageName=storageName,
                datacenter=datacenter,
                host=config.MAIN_HOST_NAME)
'''
        
###############################################################################
        
        
#################### VM POOLS #################################################
#def createVmPool(poolName, templateName, clusterName=config.MAIN_CLUSTER_NAME,
#                 size=1):
def createVmPool(poolName, 
                 templateName, 
                 clusterName, 
                 size=1):
    """ Create Vm pool """
    template = API.templates.get(templateName)
    cluster = API.clusters.get(clusterName)

    API.vmpools.add(params.VmPool(name=poolName, cluster=cluster,
                template=template, size=size))

#    ff = getFilterHeader()
#    loginAsAdmin()
    return API.vmpools.get(poolName) is not None
    LOGGER.info(timestampScale() + 'Creating vmpool "' + poolName + '"')
#    loginAsUser(filter_=ff)


def createTemplate(vmName, templateName):
    """ Create template from vmName """
#    vm = getObjectByName(API.vms, vmName)
    vm = API.vms.get(vmName)
    API.templates.add(params.Template(name=templateName, vm=vm))
    LOGGER.info(timestampScale() + 'Creating temaplate "' + templateName + '"')
    while API.vms.get(vmName).status.state != 'down':
            sleep(1)
    return True
#    waitForState(vm, states.vm.down)
#    assert getObjectByName(API.templates, templateName) is not None

#def createVm(vmName, memory=1*GB, createDisk=True, diskSize=512*MB,
#                cluster=config.MAIN_CLUSTER_NAME,
#                storage=config.MAIN_STORAGE_NAME):

def createVm(vmName,
             cluster,
             storage, 
             templateName="Blank", memory=1*GB, createDisk=True, diskSize=512*MB):
    """ Creates VM and adds a system disk.

    The defaultly created disk will be a bootable COW sparse system disk with
    size `diskSize` and interface virtio. If you want to add a different disk,
    set `createDisk` to False and add it manually.
    """
#    cluster = getObjectByName(API.clusters, cluster)
#    template = getObjectByName(API.templates, 'Blank')
    template = API.templates.get(templateName)
    cluster = API.clusters.get(cluster)
    
    # ------------------------------------------------------------------------------------
    vm = API.vms.get(vmName)
    #if getObjectByName(API.vms, vmName) is not None:
    if vm is not None:
        LOGGER.warning(timestampScale() + "VM: '%s' with this name already exists" % vmName)
        return

    # ------------------------------------------------------------------------------------
    if templateName is "Blank":
        API.vms.add(params.VM(name=vmName, memory=memory, cluster=cluster, template=template))
        
        vm = API.vms.get(vmName)
                
        if createDisk:
            LOGGER.info(timestampScale() + "Attaching disk to VM: '%s'" % vmName)
    #        sd = getObjectByName(API.storagedomains, storage)
            sd = API.storagedomains.get(storage)
            param = params.StorageDomains(
                    storage_domain=[sd])
    #        updateObject(vm).disks.add(params.Disk(
            vm.disks.add(params.Disk(
                                     storage_domains=param, size=diskSize, #type_='system',
                                     status=None, interface='virtio', format='cow',
                                     sparse=True, bootable=True))
            LOGGER.info(timestampScale() + "Attaching NIC to VM: '%s'" % vmName)
            vm.nics.add(params.NIC(
                                   name='eth0', 
                                   network=params.Network(name='rhevm'), 
                                   interface='virtio'))
            
    else:
        '''
        This means you create vm from template
        cow - thin | raw - preallocated
        Create same disk format (cow / raq) as is in template, by default it's thin  
        '''
        API.vms.add(params.VM(name=vmName, description="vm from template: " + templateName, memory=memory, cluster=cluster, template=template, disks=params.Disks(clone=True)))
            
    # ------------------------------------------------------------------------------------
#    vm = getObjectByName(API.vms, vmName)
#    vm = API.vms.get(vmName)
#    return vm is not None, "Failed to create vm"

#    waitForState(vm, states.vm.down)
    while API.vms.get(vmName).status.state != "down":
#        print API.vms.get(vmName).status.state + " <- VM status -> " + vmName
        sleep(1)
    # ------------------------------------------------------------------------------------
    currentDiskState = "unknow"

    while currentDiskState != "ok":
        allDisksVm = API.vms.get(vmName).get_disks().list()
        currentDiskState = allDisksVm[0].status.state
        sleep(1)
    LOGGER.info(timestampScale() + "Disk to VM: '%s'" % vmName + " attached")
#    print currentDiskState + " <- disk status -> " + vmName
    # ------------------------------------------------------------------------------------

    LOGGER.info(timestampScale() + "VM '%s' was created." %(vmName))
    return True

##### Find Missed VMs #####
def findMissedVM(vmNamePool):
    for i in range(len(vmNamePool)):
#        print vmNamePool[i]
        vm = API.vms.get(vmNamePool[i])
        if vm is None:
            print "VM: '%s' is not exists" % vmNamePool[i]

##### Start One VM #####
def startVM(vmName):
    vm = API.vms.get(vmName)  
    try:
        if vm.status.state != 'up':
            vm.start()
            LOGGER.info(timestampScale() + "Starting VM: '%s'" % vmName)
        else:
            print 'VM: ' + vmName + ' already up'
    except Exception as e:
        print 'Failed to Start VM:\n%s' % str(e)
    return True

##### Stop One VM #####
def stopVM(vmName):
    vm = API.vms.get(vmName)  
    try:
        if vm.status.state != 'down':
            vm.stop()
            LOGGER.info(timestampScale() + "Stoping VM: '%s'" % vmName)
        else:
            print 'VM: ' + vmName + ' already down'
    except Exception as e:
        print 'Failed to Stop VM:\n%s' % str(e)
    return True 

##### Turbo Power On / Off One VM #####
def turboStartStopOneVM(vmObject, state="ON"):
    if state != "ON":
        '''Power Off'''
        if vmObject.status.state != 'down':
            vmObject.stop()
            LOGGER.info(timestampScale() + "Turbo Stoping VM: '%s'" % vmObject.name)
        else:
            LOGGER.warning(timestampScale() + "VM: '%s' already down" % vmObject.name)        
    else:
        '''Power On'''
        if vmObject.status.state != 'up':
            vmObject.start()
            LOGGER.info(timestampScale() + "Turbo Starting VM: '%s'" % vmObject.name)
        else:
            LOGGER.warning(timestampScale() + "VM: '%s' already up" % vmObject.name)
    return True

##### Turbo Remove VM #####    
def turboRemoveVM(vmObject):
    if vmObject.status.state == 'down':
        vmObject.delete()
        LOGGER.info(timestampScale() + "Object '%s' removed." % vmObject.name)
    else:
        LOGGER.warning(timestampScale() + "Failed removed '%s' Object." % vmObject.name)
    return True

###############################################################################

from concurrent.futures import ThreadPoolExecutor



@timeIt
def runMultithreadingOperationsOneParameter(func, amount, parallel, namesOfObjects=None,**kwargs):
    '''
    Runs function in parallel using a worker pool
    '''
    threads = []
    with ThreadPoolExecutor(parallel) as threadManager:
        for i in xrange(amount):        
            if namesOfObjects is not None:
                kwargs['name']  = namesOfObjects[i]
            LOGGER.info(timestampScale() + "Calling %s, iteration %d of %d" % (func.__name__, i + 1, amount))
            threads.append(threadManager.submit(func, **kwargs))
    
    for thread in threads:
        ex = thread.exception()
        if ex is not None:
            raise ex
        result = thread.result() 
        if not result:
            raise Exception("Error during %s" % func.__name__)

    return True


@timeIt
def runMultithreadingOperations(func, amount, parallel, changingParameters, **kwargs):
    '''
    Runs function in parallel using a worker pool
    params:
    amount: amount of iterations to run of the operation
    parallel: maximum number of worker threads available in parallel 
    changingParameters: dictionary with keywords as parameter names and value is a list of values for each iteration
    kwargs: parameters passed directly to function, do not change between iterations
    returns True if all operations suceed
    throws exception if a call recieves an exception or if an iteration is not succesfull 
    '''
    threads = []
    with ThreadPoolExecutor(parallel) as threadManager:
        for i in xrange(amount):
            for keyword in changingParameters.keys():
                kwargs[keyword]  = changingParameters[keyword][i]
            LOGGER.info(timestampScale() + "Calling %s, iteration %d of %d" % (func.__name__, i + 1, amount))
            threads.append(threadManager.submit(func, **kwargs))
    
    for thread in threads:
        ex = thread.exception()
        if ex is not None:
            raise ex
        result = thread.result() 
        if not result:
            raise Exception("Error during %s" % func.__name__)

    return True

    
    

#def looper(amountOfObjects, names):
#    for i in xrange(amountOfObjects):
#        createDataCenter(name=names[i])